part of fields.view;

final Tag fieldListView = createTag((_) => new FieldListView());

class FieldListViewResources {
  String get style =>
"""
table {
  border-collapse:collapse;
  border-top:1px solid ${fieldsTheme.colors.border};
  border-left:1px solid ${fieldsTheme.colors.border};
  margin:-1px;            
  padding:0px;
  table-layout:fixed;
  width:100%;
}
td {
  background-color: ${fieldsTheme.colors.fieldBackground};
  border-top:1px solid ${fieldsTheme.colors.border};
  border-left:1px solid ${fieldsTheme.colors.border};
  color: ${fieldsTheme.colors.text};
  vertical-align:text-top;
  margin:0px;
  overflow:hidden;
  padding:0px;
  text-overflow:ellipsis;
}
.namebox {
  padding-left:4px;
  width:20%;
}
""";
  const FieldListViewResources();
}

class FieldListView extends HtmlElement {
  
  static const TAG = 'field-list-view';
  
  final Map<Field, TableRowElement> _cache = new Map<Field, TableRowElement>();
  Element _tbody;
  
  factory FieldListView() => new Element.tag(TAG);
  
  FieldListView.created() : super.created() {
    FieldListViewResources res = const FieldListViewResources();
    children(this, () {
      span(() {
        style()
        ..scoped = true
        ..text = res.style;
        table(() => _tbody = tbody());
      });
    });
  }

  void clearCache() => _cache.clear();
  
  void render(List<Field> fields) {
    for (Field f in _cache.keys.where((f) => !fields.contains(f))) {
      _tbody.children.remove(_cache[f]);
    }
    for (int i = 0; i < fields.length; i++) {
      Field f = fields[i];            
      if (_cache.containsKey(f)) {
        final Element el = _cache[f];
        _insertChildAt(el, i);
        final FieldView fv = el.children[1].firstChild;
        fv.render();
      } else {
        final el = tr(() {
          td(() => span()..text = f.displayName)
          ..title = f.displayName
          ..classes = ['namebox'];
          td(() => fieldView(null, f));
        });
        _cache[f] = el;
        _insertChildAt(el, i);
      }
    }
  }
  
  void _insertChildAt(Element element, int index) {
    if (index < _tbody.children.length) {
      _tbody.insertBefore(element, _tbody.children[index]);
    } else {
      _tbody.append(element);
    }
  }
}
