part of fields.view;

final Tag fieldView = createTag((Field f) => new FieldView(f));

class FieldViewResources {
  String get style =>
"""
.expander {
  background-color:transparent;
  border-style:none;
  color:${fieldsTheme.colors.text};
  float:left;   
  width:16px;
  padding:0;
  margin-top:4px;
}
.editorbox {
  display:block; 
  overflow:hidden;  
}
""";
  const FieldViewResources();
}

class FieldView extends HtmlElement {  
  
  static const TAG = 'field-view';
  
  final FieldListView _listView = new FieldListView();  
  ToggleButton _toggle;
  
  Field _field;
  Field get field => _field;
  
  FieldEditor _editor;  
  bool get _isExpanded => _toggle.isDown;
  
  factory FieldView(Field field) {    
    final FieldView fv = new Element.tag(TAG);
    fv._init(field);
    return fv;
  }
  
  FieldView.created() : super.created() {}
  
  void _init(Field field, 
             [FieldViewResources res = const FieldViewResources()]) {
    
    _field = field;
    _editor = new FieldEditor.forField(_field);
    
    children(this, () {
      shadow(() {
        style()..text = res.style;            
        _toggle = toggleButton(() {
          span()..innerHtml = '&#9655;'..classes = ['up'];
          span()..innerHtml = '&#9698;'..classes = ['down'];
        })
        ..classes.add('expander')
        ..onToggle.listen((_) => render());
        span(() {
          append(_editor.element);
        })
        ..classes.add('editorbox');
        append(_listView);
      });
    });
    
    field.values.listen((_) => _render());
    _render();
  }
  
  void render() {
    if (_isExpanded) {
      _listView.render(_field.children);
    } else {
      _listView.render([]);
    }
    _render();
  }
  
  void _render() {
    _toggle.hidden = _field.children.isEmpty;
    _listView.hidden = !_isExpanded;    
  }
}
