part of fields.view;

final Tag objectEditor = createTag((_) => new ObjectEditor());

class ObjectEditorResources {
  String get style =>
"""
@host {
  * {
    background-color: ${fieldsTheme.colors.background};
    border:1px solid ${fieldsTheme.colors.border};
    height:100%;    
    overflow:hidden;
    width:100%;
  }
}
.toggle {
  background-color:transparent;
  border-style:none;
  float:left;
}
.toggle:disabled {
  opacity:0.4;
}
.filterbox {  
  display:block; 
  overflow:hidden;
}
.filter {
  background-color:transparent;
  border-style:none;
  color:${fieldsTheme.colors.text};
  width:100%;
}
""";

  String get down => fieldsTheme.images.down;
  String get up => fieldsTheme.images.up;
  String get lock => fieldsTheme.images.lock;
  String get unlock => fieldsTheme.images.unlock;  
  const ObjectEditorResources();
}

class ObjectEditor extends HtmlElement {
  
  static const TAG = 'object-editor';
  
  String _filterText = '';  
  // TODO(rms): investigate the runtime error we get here with dart2js
//  Predicate _filter;
  Predicate _filterPrivate;
    
  ButtonElement _sortButton;
  ButtonElement _lockButton;
  InputElement _filterBox;
  
  FieldList _list;
  FieldListView _listView;
  StreamSubscription _listListener;
  
  bool get hasSelected => _selected != null;
      
  Object _selected;
  Object get selected => _selected;
  set selected(Object value) {           
    if(value != selected) {
      if (_listListener != null) _listListener.cancel();      
      _selected = value;
      _list = new FieldList()
      ..refreshSource(_selected)
      ..predicate = _filterPrivate;      
      _listListener = _list.fieldChanged.listen((_) {
        _selectedChanged.add(_selected);
      });
      _listView.clearCache();
      _render();
      _selectedChanged.add(_selected);
    }
  }
  
  final _selectedChanged = new StreamController<Object>.broadcast();
  Stream<Object> get selectedChanged => _selectedChanged.stream;
  
  factory ObjectEditor() => new Element.tag(TAG);
  
  ObjectEditor.created() : super.created() {
    ObjectEditorResources res = const ObjectEditorResources();
            
    final _filter = (Field f) => f.displayName.contains(_filterText);
    _filterPrivate = (Field f) => !f.isPrivate && _filter(f);
    
    children(this, () {
      div(() {
        shadow(() {
          style()..text = res.style;
          _sortButton = toggleButton(() {
            img()..src = res.up..classes = ['up'];
            img()..src = res.down..classes = ['down'];            
          })
          ..classes = ['toggle']
          ..disabled = true
          ..onToggle.listen((bool isDown) {
            if (isDown) {
              _list.comparator = 
                  (Field a, Field b) => 
                      a.displayName.compareTo(b.displayName) * -1;
            } else {
              _list.comparator = FieldList.defaultComparator;
            }
            _render();
          });
          _lockButton = toggleButton(() {
            img()..src = res.lock..classes = ['up'];
            img()..src = res.unlock..classes = ['down'];
          })
          ..classes = ['toggle']
          ..disabled = true
          ..onToggle.listen((bool isDown) {
            if (isDown) {
              _list.predicate = _filter;
            } else {
              _list.predicate = _filterPrivate;
            }
            _render();
          });
          span(() {
            _filterBox = input()
            ..type = 'text'
            ..classes = ['filter']
            ..disabled = true
            ..placeholder = 'filter fields';
          })
          ..classes = ['filterbox']
          ..onInput.listen((_) {
            _filterText = _filterBox.value;
            _list.refreshView();
            _render();
          });
          _listView = fieldListView();
        });
      });
    });
    
    registerDefaultFieldTransforms();
    registerDefaultFieldEditors();
  }
  
  void refresh() {
    if (hasSelected) {
      _list.refreshSource(_selected);
      _render();
    }
  }
  
  void _render() { 
    _sortButton.disabled = !hasSelected;
    _lockButton.disabled = !hasSelected;
    _filterBox.disabled = !hasSelected;
    _listView.render(_list);
  }
}
